/*
 *  ngtk/include/ngtk.h
 *
 *  Copyright (C) 1998, 1999 Gabriel Fort <gabx@tfz.net>
 *
 */

/*----------------------------------------------------------------------------

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the 
    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
    Boston, MA  02111-1307  USA.

----------------------------------------------------------------------------*/
#ifndef _NGTK_H
#define _NGTK_H     1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <ncurses.h>
#include <panel.h>

#ifdef HAVE_SLANG
#include <slang/slang.h>
#endif

/*--------------------------- Version Info ---------------------------------*/
#define NGTK_MAJOR          "0"
#define NGTK_MINOR          "06"
#define NGTK_PATCHLEVEL     "0"

#ifdef linux
    #define NGTK_OS         "Linux"
#else
    #define NGTK_OS         "Unknown"
#endif

/*------------------------ Macros Definitions ------------------------------*/
#define MAX_USER_FUNCTIONS  64  /* max number of mapped functions */
#define T_BAR               16  /* max number of entries in footbar */
#define MAX_KEYSYMS         32  /* max number of redefinable keysyms */
                                /* these 3 limitations shouldn't exitst... */

#define __ERR__             "__RAHHHH__"    /* error string ;-) */


/* Basic color codes */
#define     BLACK_C               0
#define     RED_C                 1
#define     GREEN_C               2
#define     YELLOW_C              3
#define     BLUE_C                4
#define     MAGENTA_C             5
#define     CYAN_C                6
#define     WHITE_C               7

/* Color pairs definition macros: use XXX_F(YYY_B) to expand to the number of
   a color pair with XXX as foreground color, and YYY as background color */
#define     BLACK_B             0       /* Background Colors */
#define     RED_B               8       /* you can also use those alone to */
#define     GREEN_B             16      /* get a color pair with black */
#define     YELLOW_B            24      /* foreground */
#define     BLUE_B              32
#define     MAGENTA_B           40
#define     CYAN_B              48
#define     WHITE_B             56

#define     BLACK_F(x)          x + BLACK_C   /* Foreground Colors */
#define     RED_F(x)            x + RED_C
#define     GREEN_F(x)          x + GREEN_C
#define     YELLOW_F(x)         x + YELLOW_C
#define     BLUE_F(x)           x + BLUE_C
#define     MAGENTA_F(x)        x + MAGENTA_C
#define     CYAN_F(x)           x + CYAN_C
#define     WHITE_F(x)          x + WHITE_C

/* Key definitions */
#ifndef HAVE_SLANG
    #define NGTK_KEY_UP     0x101
    #define NGTK_KEY_DOWN   0x102
    #define NGTK_KEY_LEFT   0x103
    #define NGTK_KEY_RIGHT  0x104
    #define NGTK_KEY_PPAGE  0x105
    #define NGTK_KEY_NPAGE  0x106
    #define NGTK_KEY_HOME   0x107
    #define NGTK_KEY_END    0x108
    #define NGTK_KEY_BACKSPACE  0x110
    #define NGTK_KEY_ENTER  0x111
    #define NGTK_KEY_IC     0x112
    #define NGTK_KEY_DELETE 0x113
    #define NGTK_KEY_F0     0x200
    #define NGTK_KEY_F(x)   (NGTK_KEY_F0 + x)

    #define CR              0x0D
    #define ESC             0x1B
#else
    #define NGTK_KEY_UP     SL_KEY_UP
    #define NGTK_KEY_DOWN   SL_KEY_DOWN
    #define NGTK_KEY_LEFT   SL_KEY_LEFT
    #define NGTK_KEY_RIGHT  SL_KEY_RIGHT
    #define NGTK_KEY_PPAGE  SL_KEY_PPAGE
    #define NGTK_KEY_NPAGE  SL_KEY_NPAGE
    #define NGTK_KEY_HOME   SL_KEY_HOME
    #define NGTK_KEY_END    SL_KEY_END
    #define NGTK_KEY_BACKSPACE SL_KEY_BACKSPACE
    #define NGTK_KEY_ENTER  SL_KEY_ENTER
    #define NGTK_KEY_IC     SL_KEY_IC
    #define NGTK_KEY_DELETE SL_KEY_DELETE
    #define NGTK_KEY_F0     SL_KEY_F0
    #define NGTK_KEY_F(x)   SL_KEY_F(x)

    #define CR              0x0D
    #define ESC             0x1B
#endif

/* SLang vs. Ncurses macros */
#ifdef HAVE_SLANG
    #define     ngtk_init_input()   ngtk_init_slang()
    #define     ngtk_tty_getc()     ngtk_tty_getc_slang()
    #define     ngtk_define_keysym(a, b)    SLkp_define_keysym(a, b)
#else
    #define     ngtk_init_input()   ngtk_init_ncurses_key()
    #define     ngtk_tty_getc()     ngtk_tty_getc_ncurses()
    #define     ngtk_define_keysym(a, b)    ngtk_define_keysym_ncurses(a, b)
#endif

/*--------------------------- Types Definitions ----------------------------*/

/* --------------- Input Types -----*/
/* Escape Sequence <=> Keysym */
typedef struct {
    int keysym;
    char *esc_seq;
} ngtk_esc_key_t;

/* Keysym <=> Function */
typedef struct {
    int keysym;
    void (*func)(void);     /* Pointer to the corresponding function */
} ngtk_stroke_t;


/* --------------- Output Types -----*/
/* Ngtk Window Widget, the base Widget of Ngtk */

typedef struct _ngtk_win_t ngtk_win_t;

struct _ngtk_win_t {
    WINDOW *win;            /* Pointer to the corresponding ncurses window */
    WINDOW *shadow;         /* Pointer to the window's shadow, if set */
    PANEL *pan;             /* Pointer to the corresponding panel */
    PANEL *shadow_pan;      /* Pointer to the shadow's panel, if set */
    int cursor_flag;        /* Echo the cursor if set, keep it invisble if 0 */
    int shadow_flag;        /* Window shadowed if set, not shadowed if 0 */
    int cpair;              /* Color pair of the window */
    int attr;               /* Attribute of the window */
    ngtk_win_t *prev;
};


/* Radio Widget */

typedef struct {
    char *text;
    int ret;
} ngtk_radio_item;

typedef struct {
    ngtk_win_t *container;
    ngtk_win_t *box;

    char *title;
    int item_count;
    int max_item_len;
    int current_item;
    int prev_item;

    int boxed;
    int visible;

    ngtk_radio_item *items;
} ngtk_radio_t;

/* Menu Item */
typedef struct {
    char *text;             /* Text of the item */
    void (*func)(void);     /* Pointer to the function linked with the item */
} ngtk_menu_item;


/* Menu Widget */
typedef struct {
    ngtk_win_t *container;  /* Container window */
    ngtk_win_t *box;        /* Box Window (optional) */

    char *title;            /* Box Title (optional) */
    int item_count;         /* Items count for the menu */
    int max_item_len;       /* Length of the longest item text string */
    int fcpair;             /* foreground color pair (selected item) */
    int bcpair;             /* background color pair (the rest of the win) */
    int fattr;              /* foreground attribute */
    int battr;              /* background attribute */
    int top;                /* Last coordinates of the displayed menu */
    int left;               /*                                        */

    int boxed;              /* set if menu contained in a box */
    int visible;            /* set if menu is visible, 0 if not */
    int current_item;       /* currently selected item */
    int previous_item;      /* check if the previous item was above or under */
    int clear_flag;         /* destroy the menu ? */

    ngtk_menu_item *items;  /* pointer to the items definitions array */
} ngtk_menu_t;


/* Progress Bar Widget */
typedef struct {
    ngtk_win_t *container;  /* container window */
    int width;              /* Bar Width */
    int percent;            /* Current percentage */
} ngtk_pbar_t;

/* Scrolling Line */
typedef struct _ngtk_scline_t {
    struct _ngtk_scline_t *prev;
    struct _ngtk_scline_t *next;
    char *str;
} ngtk_scline_t;

/* Scrolling Area Widget */
typedef struct {
    ngtk_win_t *container;      /* window containing the scroll */
    ngtk_scline_t **lines;      /* array of all the lines */
    int cur_index;
    int nlines;                 /* total number of lines */
} ngtk_scroll_t;

                    /* NOT EVEN YET ALPHA CODE BEGIN */
typedef struct {
    ngtk_win_t *container;
    int direction;              /* 0-> horizontal ; 1-> vertical */
    int dim;                    /* height or width, depending on direction */
    int percent;
    int cursor_dim;             /* size of the cursor sliding on the bar, */
} ngtk_sbar_t;                  /* calculated from the bar's size */

/* --------------- Input/Output Types -----*/
typedef struct {
    ngtk_win_t *container;      /* container window */
    int status;                 /* 0 => button clear; 1 => button set */

} ngtk_button_t;
                    /* NOT EVEN YET ALPHA CODE END */


/*--------------------------- Global Variables -----------------------------*/
/* keysym <=> function array */
ngtk_stroke_t functable[MAX_USER_FUNCTIONS];    /* these 2 objects have */
                                                /* to be improved... */
/* escape sequence <=> keysym array */
ngtk_esc_key_t keysyms[MAX_KEYSYMS];

/* last index to the keysyms[] array */
int last_keysym;
int key_table;

/* Buffer storing the result of a ngtk_tty_getc_ncurses (too long ?) */
char keybuf[1024];

/* Vars used in ngtk_footbar */
char t_bar1[T_BAR][1024];   /* Strings to be displayed in the 1st color */
char t_bar2[T_BAR][1024];   /* Strings to be displayed in the 2nd color */
int t_bar;                  /* Number of members used in t_bar1 & t_bar2 */

ngtk_win_t *prev_buffer;
int prev_curs;

ngtk_win_t *background;
ngtk_win_t *top_bar;
ngtk_win_t *foot_bar;

int use_ngtk_main;

/*------------------------------- Prototypes ---------------------------------
    Prototypes for ngtk_bars.c
----------------------------------------------------------------------------*/
extern void ngtk_footbar(int cpair1, int attr1, int cpair2, int attr2);
extern void ngtk_topbar(int cpair, int attr, char *msg, int init);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_boxes.c
----------------------------------------------------------------------------*/
extern void ngtk_minibox(ngtk_win_t *cur_win, int height, int width, int top,\
                         int left, int show, int up, int refresh);
extern ngtk_win_t *ngtk_mkbox(int height, int width, int top, int left, \
                              int cpair, int attr, char *title, int echo, \
                              int shadow, int refresh);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_fade.c
----------------------------------------------------------------------------*/
extern void ngtk_fade_in(void);
extern void ngtk_fade_out(void);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_getxtbox.c
----------------------------------------------------------------------------*/
extern void ngtk_getxtbox(int height, int width, int top, int left,
                          int bandlen, int virtlen,
                          int boxpair, int boxattr, int textpair,
                          int textattr, int prtpair1, int prtattr1,
                          int prtpair2, int prtattr2, char *title,
                          char *prompt, char *str);
extern void ngtk_auto_pos_getxtbox(int height, int width, int bandlen, int virtlen,
                                   int boxpair, int boxattr, int textpair,
                                   int textattr, int prtpair1, int prtattr1,
                                   int prtpair2, int prtattr2, char *title,
                                   char *prompt, char *str);
extern void ngtk_auto_size_getxtbox(int top, int left, int bandlen, int virtlen,
                                    int boxpair, int boxattr, int textpair,
                                    int textattr, int prtpair1, int prtattr1,
                                    int prtpair2, int prtattr2, char *title,
                                    char *prompt, char *str);
extern void ngtk_auto_getxtbox(int bandlen, int virtlen, int boxpair, int boxattr,
                               int textpair, int textattr, int prtpair1,
                               int prtattr1, int prtpair2, int prtattr2,
                               char *title, char *prompt, char *str);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_init.c
----------------------------------------------------------------------------*/
extern void ngtk_launch_display(void);
extern int ngtk_stop_display(void);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_input.c
----------------------------------------------------------------------------*/
extern void ngtk_init_ncurses_key(void);
extern size_t ngtk_tty_input(char *buf, size_t length);
extern void ngtk_tty_read(void);
extern int ngtk_parse_buffer(void);
extern int ngtk_tty_getc_ncurses(void);
extern int ngtk_tty_getc_slang(void);
extern int ngtk_define_keysym_ncurses(char *esc_seq, int keysym);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_input_thread.c
----------------------------------------------------------------------------*/
extern int ngtk_get_keysym(void);
extern void ngtk_main(void);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_menus.c
----------------------------------------------------------------------------*/
extern ngtk_menu_item *ngtk_mk_menu_items(const int n_its,
                                          const char *texts[n_its],
                                          const void *funcs[n_its]);
extern ngtk_menu_t *ngtk_mk_menu(const char *title, const int item_count,
                                 const char *texts[item_count],
                                 const void *funcs[item_count]);
extern ngtk_menu_t *ngtk_show_menu(ngtk_menu_t *c_menu, int top, int left,
                                   int bcpair, int battr, int fcpair, int fattr, int box);
extern ngtk_menu_t *ngtk_hide_menu(ngtk_menu_t *c_menu);
extern void ngtk_move_selection(ngtk_menu_t *c_menu);
extern void *ngtk_manage_menu(ngtk_menu_t *c_menu);
extern void ngtk_destroy_menu(ngtk_menu_t *c_menu);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_omnibox.c
----------------------------------------------------------------------------*/
extern void ngtk_omnibox(int height, int width, int top, int left, int cpair,
                         int attr, char *title, int nlines, char *msg[nlines]);
extern void ngtk_auto_pos_omnibox(int height, int width, int cpair, int attr,
                                  char *title, int nlines, char *msg[nlines]);
extern void ngtk_auto_size_omnibox(int top, int left, int cpair, int attr,
                                   char *title, int nlines, char *msg[nlines]);

extern void ngtk_auto_omnibox(int cpair, int attr, char *title, int nlines,
                              char *msg[nlines]);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_pbar.c
----------------------------------------------------------------------------*/
extern ngtk_pbar_t *ngtk_mk_pbar(int width);
extern void ngtk_show_pbar(ngtk_pbar_t *c_pbar, int top, int left, int cpair,
                           int attr);
extern void ngtk_update_pbar(ngtk_pbar_t *c_pbar, int percentage);
extern void ngtk_destroy_pbar(ngtk_pbar_t *c_pbar);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_radio.c
----------------------------------------------------------------------------*/
extern ngtk_radio_item *ngtk_mk_radio_items(int n_items, char *list[n_items],
                                            int rets[n_items]);
extern ngtk_radio_t *ngtk_mk_radio(char *title, int n_items, char *list[n_items-1],
                                   int rets[n_items-1]);
extern ngtk_radio_t *ngtk_show_radio(ngtk_radio_t *c_radio, int height, int width,
                                     int top, int left, int cpair, int attr, int box,
                                     int def_item);
extern ngtk_radio_t *ngtk_hide_radio(ngtk_radio_t *c_radio);
extern ngtk_radio_t *ngtk_manage_radio(ngtk_radio_t *c_radio);
extern void ngtk_destroy_radio(ngtk_radio_t *c_radio);
extern ngtk_radio_t *ngtk_auto_pos_show_radio(ngtk_radio_t *c_radio, int height,
                                              int width, int cpair, int attr,
                                              int box, int def_item);
extern ngtk_radio_t *ngtk_auto_size_show_radio(ngtk_radio_t *c_radio, int top,
                                               int left, int cpair, int attr,
                                               int box, int def_item);
extern ngtk_radio_t *ngtk_auto_show_radio(ngtk_radio_t *c_radio, int cpair, int attr,
                                          int box, int def_item);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_readline.c
----------------------------------------------------------------------------*/
extern void ngtk_virt_readline(WINDOW *win, int vislen, int virtlen, char *str);
extern void ngtk_readline(WINDOW *win, int maxlen, char *str);
extern void ngtk_virt_read_win(int len, int virtlen, int top, int left, \
                               int cpair, int attr, char *str);
extern void ngtk_read_win(int len, int top, int left, int cpair, int attr, \
                          char *str);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_scroll.c
----------------------------------------------------------------------------*/
extern ngtk_scline_t *ngtk_mk_scroll_line(char *str, int padding);
extern void ngtk_free_scroll_line(ngtk_scline_t *line);
extern ngtk_scroll_t *ngtk_mk_scroll_buffer(ngtk_win_t *container, int nlines, \
                                            char *strs[nlines+1]);
extern void ngtk_free_scroll_buffer(ngtk_scroll_t *buffer);
extern ngtk_scroll_t *ngtk_mk_scroll_win(int height, int width, int top, \
                                         int left, int cpair, int attr, \
                                         int echo_cursor, int shadow, \
                                         int nlines, char *strs[nlines+1]);
extern void ngtk_display_scroll_buffer(ngtk_scroll_t *buffer, int topline);
extern void ngtk_scroll_line_up(ngtk_scroll_t *buffer);
extern void ngtk_scroll_line_down(ngtk_scroll_t *buffer);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_slang.c
----------------------------------------------------------------------------*/
extern void ngtk_init_slang(void);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_stdbox.c
----------------------------------------------------------------------------*/
extern void ngtk_commonbox(int cpair, int attr, char *title, char *prompt,
                           char *msg);
extern void ngtk_errbox(char *errmsg);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_strings.c
----------------------------------------------------------------------------*/
extern char *ngtk_mktab(int tablen);
extern size_t ngtk_strpad(char *str, int strln);
extern void ngtk_inschar(char *str, int strln, char c, int offset);
extern void ngtk_delchar(char *str, int strln, int offset);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_utils.c
----------------------------------------------------------------------------*/
extern void ngtk_dumb_func(void);
extern void ngtk_clrwin(WINDOW *win, chtype pat, int refresh);
extern void ngtk_clreol(WINDOW *win, char pat, int refresh);
extern void ngtk_init_pairs(void);
extern void setfg(ngtk_win_t *win, int color, int attr);
extern void ngtk_register_key(int keysym, void (*func)(), char *keydsc,
                              char *funcdsc, int disp_foot);
extern int ngtk_wscrl(WINDOW *win, int n);
extern void ngtk_dccprintf(ngtk_win_t *win, int c1, int a1, int c2, int a2, \
                           char *str);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_winstack.c
----------------------------------------------------------------------------*/
extern void ngtk_move_win(ngtk_win_t *move_win, int top, int left);
extern void ngtk_pull_win(ngtk_win_t *pull_win);
extern ngtk_win_t *ngtk_push_win(int height, int width, int top, int left, int cpair,
                                 int attr, int echo_cursor, int shadow);
extern void ngtk_pop_win(ngtk_win_t *kill_win);

/*----------------------------------------------------------------------------
    Prototypes for ngtk_yesnobox.c
----------------------------------------------------------------------------*/
extern int ngtk_yesnobox(int height, int width, int top, int left, int cpair,
                         int attr, char *title, char *prompt, int def_val);
extern int ngtk_auto_pos_yesnobox(int height, int width, int cpair, int attr,
                                  char *title, char *prompt, int def_val);
extern int ngtk_auto_size_yesnobox(int top, int left, int cpair, int attr,
                                   char *title, char *prompt, int def_val);
extern int ngtk_auto_yesnobox(int cpair, int attr, char *title, char *prompt,
                              int def_val);

#endif /* !_NGTK_H */
